@@ -3,14 +3,7 @@
 use deno_ast::ParsedSource;
 use serde::Deserialize;
 use serde::Serialize;
-use std::fmt::Display;
-use std::fmt::Formatter;
-use std::fmt::Result as FmtResult;
 
-use crate::colors;
-use crate::display::display_optional;
-use crate::display::display_readonly;
-use crate::display::SliceDisplayer;
 use crate::function::FunctionDef;
 use crate::js_doc::JsDoc;
 use crate::params::ts_fn_param_to_param_def;
@@ -25,6 +18,19 @@ use crate::DocNode;
 use crate::Location;
 use crate::ParamDef;
 
+cfg_if! {
+  if #[cfg(feature = "rust")] {
+    use crate::colors;
+    use crate::display::display_optional;
+    use crate::display::display_readonly;
+    use crate::display::SliceDisplayer;
+
+    use std::fmt::Display;
+    use std::fmt::Formatter;
+    use std::fmt::Result as FmtResult;
+  }
+}
+
 #[derive(Debug, Serialize, Deserialize, Clone)]
 #[serde(rename_all = "camelCase")]
 pub struct InterfaceMethodDef {
@@ -51,11 +57,13 @@ impl From<InterfaceMethodDef> for DocNode {
         is_async: false,
         is_generator: false,
         type_params: def.type_params,
+        decorators: vec![],
       },
     )
   }
 }
 
+#[cfg(feature = "rust")]
 impl Display for InterfaceMethodDef {
   fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
     write!(
@@ -100,6 +108,7 @@ impl From<InterfacePropertyDef> for DocNode {
   }
 }
 
+#[cfg(feature = "rust")]
 impl Display for InterfacePropertyDef {
   fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
     write!(
@@ -123,6 +132,7 @@ pub struct InterfaceIndexSignatureDef {
   pub ts_type: Option<TsTypeDef>,
 }
 
+#[cfg(feature = "rust")]
 impl Display for InterfaceIndexSignatureDef {
   fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
     write!(
@@ -195,7 +205,7 @@ pub fn expr_to_name(expr: &deno_ast::swc::ast::Expr) -> String {
         Lit::JSXText(JSXText { ref raw, .. }) => raw.to_string(),
       }
     }
-    _ => "<TODO>".to_string(),
+    _ => "[UNSUPPORTED]".to_string(),
   }
 }
 
