@@ -24,7 +24,6 @@ use deno_ast::ParsedSource;
 use deno_graph::MediaType;
 use deno_graph::ModuleGraph;
 use deno_graph::ModuleSpecifier;
-use deno_graph::Resolved;
 use deno_graph::SourceParser;
 
 use std::collections::HashMap;
@@ -177,7 +176,7 @@ impl<'a> DocParser<'a> {
     for specifier in by_src.keys() {
       let resolved_specifier = self
         .graph
-        .resolve_dependency(specifier, referrer)
+        .resolve_dependency(specifier, referrer, true)
         .ok_or_else(|| DocError::Resolve(specifier.clone()))?;
       let doc_nodes = self.parse_with_reexports(resolved_specifier)?;
       let reexports_for_specifier = by_src.get(specifier).unwrap();
@@ -249,7 +248,7 @@ impl<'a> DocParser<'a> {
         ))
       })?;
 
-    let module = if let Some((_, Resolved::Specifier(types_specifier, _))) =
+    let module = if let Some((_, Some(Ok((types_specifier, _))))) =
       &module.maybe_types_dependency
     {
       self
@@ -326,7 +325,7 @@ impl<'a> DocParser<'a> {
 
           let resolved_specifier = self
             .graph
-            .resolve_dependency(&src, referrer)
+            .resolve_dependency(&src, referrer, true)
             .ok_or_else(|| DocError::Resolve(src.clone()))?;
           let import_def = ImportDef {
             src: resolved_specifier.to_string(),
