@@ -1,52 +1,28 @@
 // Copyright 2020-2021 the Deno authors. All rights reserved. MIT license.
+
 use crate::colors;
-use crate::display::{display_readonly, SliceDisplayer};
+use crate::display::display_readonly;
+use crate::display::SliceDisplayer;
 use crate::interface::expr_to_name;
 use crate::params::ts_fn_param_to_param_def;
 use crate::ts_type_param::maybe_type_param_decl_to_type_param_defs;
 use crate::ts_type_param::TsTypeParamDef;
 use crate::ParamDef;
-use deno_ast::swc::ast::{
-  BigInt, Bool, CallExpr, Expr, ExprOrSuper, Lit, NewExpr, Number, Str, Tpl,
-  TplElement, TsArrayType, TsConditionalType, TsExprWithTypeArgs,
-  TsFnOrConstructorType, TsIndexedAccessType, TsKeywordType, TsLit, TsLitType,
-  TsOptionalType, TsParenthesizedType, TsRestType, TsThisType,
-  TsThisTypeOrIdent, TsTupleType, TsType, TsTypeAnn, TsTypeLit, TsTypeOperator,
-  TsTypeParamInstantiation, TsTypePredicate, TsTypeQuery, TsTypeRef,
-  TsUnionOrIntersectionType, VarDeclarator,
-};
-use serde::{Deserialize, Serialize};
-use std::fmt::{Display, Formatter, Result as FmtResult};
 
-// pub enum TsType {
-//  *      TsKeywordType(TsKeywordType),
-//  *      TsThisType(TsThisType),
-//  *      TsFnOrConstructorType(TsFnOrConstructorType),
-//  *      TsTypeRef(TsTypeRef),
-//  *      TsTypeQuery(TsTypeQuery),
-//  *      TsTypeLit(TsTypeLit),
-//  *      TsArrayType(TsArrayType),
-//  *      TsTupleType(TsTupleType),
-//  *      TsOptionalType(TsOptionalType),
-//  *      TsRestType(TsRestType),
-//  *      TsUnionOrIntersectionType(TsUnionOrIntersectionType),
-//  *      TsConditionalType(TsConditionalType),
-//  *      TsParenthesizedType(TsParenthesizedType),
-//  *      TsTypeOperator(TsTypeOperator),
-//  *      TsIndexedAccessType(TsIndexedAccessType),
-//  *      TsLitType(TsLitType),
-//     TsInferType(TsInferType),
-//     TsMappedType(TsMappedType),
-//  *      TsTypePredicate(TsTypePredicate),
-//     TsImportType(TsImportType),
-// }
+use deno_ast::swc::ast::*;
+use deno_ast::swc::common::Span;
+use serde::Deserialize;
+use serde::Serialize;
+use std::fmt::Display;
+use std::fmt::Formatter;
+use std::fmt::Result as FmtResult;
 
 impl From<&TsLitType> for TsTypeDef {
   fn from(other: &TsLitType) -> TsTypeDef {
     match &other.lit {
       TsLit::Number(num) => (TsTypeDef::number_literal(num)),
       TsLit::Str(str_) => (TsTypeDef::string_literal(str_)),
-      TsLit::Tpl(tpl) => TsTypeDef::tpl_literal(&tpl.quasis),
+      TsLit::Tpl(tpl) => TsTypeDef::tpl_literal(&tpl.types, &tpl.quasis),
       TsLit::Bool(bool_) => (TsTypeDef::bool_literal(bool_)),
       TsLit::BigInt(bigint_) => (TsTypeDef::bigint_literal(bigint_)),
     }
@@ -337,6 +313,30 @@ impl From<&TsIndexedAccessType> for TsTypeDef {
   }
 }
 
+impl From<&TsMappedType> for TsTypeDef {
+  fn from(other: &TsMappedType) -> Self {
+    let mapped_type_def = TsMappedTypeDef {
+      readonly: other.readonly,
+      type_param: Box::new((&other.type_param).into()),
+      name_type: other
+        .name_type
+        .as_ref()
+        .map(|nt| Box::new(TsTypeDef::from(&**nt))),
+      optional: other.optional,
+      ts_type: other
+        .type_ann
+        .as_ref()
+        .map(|a| Box::new(TsTypeDef::from(&**a))),
+    };
+
+    TsTypeDef {
+      mapped_type: Some(mapped_type_def),
+      kind: Some(TsTypeDefKind::Mapped),
+      ..Default::default()
+    }
+  }
+}
+
 impl From<&TsTypeLit> for TsTypeDef {
   fn from(other: &TsTypeLit) -> TsTypeDef {
     let mut methods = vec![];
@@ -508,6 +508,50 @@ impl From<&TsConditionalType> for TsTypeDef {
   }
 }
 
+impl From<&TsInferType> for TsTypeDef {
+  fn from(other: &TsInferType) -> Self {
+    let infer = TsInferDef {
+      type_param: Box::new((&other.type_param).into()),
+    };
+
+    Self {
+      kind: Some(TsTypeDefKind::Infer),
+      infer: Some(infer),
+      ..Default::default()
+    }
+  }
+}
+
+impl From<&TsImportType> for TsTypeDef {
+  fn from(other: &TsImportType) -> Self {
+    let type_params = if let Some(type_params_inst) = &other.type_args {
+      let mut ts_type_defs = vec![];
+
+      for type_box in &type_params_inst.params {
+        let ts_type: &TsType = &(*type_box);
+        let def: TsTypeDef = ts_type.into();
+        ts_type_defs.push(def);
+      }
+
+      Some(ts_type_defs)
+    } else {
+      None
+    };
+
+    let import_type_def = TsImportTypeDef {
+      specifier: other.arg.value.to_string(),
+      qualifier: other.qualifier.as_ref().map(|e| ts_entity_name_to_name(e)),
+      type_params,
+    };
+
+    Self {
+      kind: Some(TsTypeDefKind::ImportType),
+      import_type: Some(import_type_def),
+      ..Default::default()
+    }
+  }
+}
+
 impl From<&TsFnOrConstructorType> for TsTypeDef {
   fn from(other: &TsFnOrConstructorType) -> TsTypeDef {
     use deno_ast::swc::ast::TsFnOrConstructorType::*;
@@ -565,26 +609,26 @@ impl From<&TsType> for TsTypeDef {
     use deno_ast::swc::ast::TsType::*;
 
     match other {
-      TsKeywordType(ref keyword_type) => keyword_type.into(),
-      TsLitType(ref lit_type) => lit_type.into(),
-      TsTypeRef(ref type_ref) => type_ref.into(),
-      TsUnionOrIntersectionType(union_or_inter) => union_or_inter.into(),
-      TsArrayType(array_type) => array_type.into(),
-      TsTupleType(tuple_type) => tuple_type.into(),
-      TsTypeOperator(type_op_type) => type_op_type.into(),
-      TsParenthesizedType(paren_type) => paren_type.into(),
-      TsRestType(rest_type) => rest_type.into(),
-      TsOptionalType(optional_type) => optional_type.into(),
-      TsTypeQuery(type_query) => type_query.into(),
+      TsKeywordType(keyword_type) => keyword_type.into(),
       TsThisType(this_type) => this_type.into(),
       TsFnOrConstructorType(fn_or_con_type) => fn_or_con_type.into(),
-      TsConditionalType(conditional_type) => conditional_type.into(),
-      TsIndexedAccessType(indexed_access_type) => indexed_access_type.into(),
+      TsTypeRef(type_ref) => type_ref.into(),
+      TsTypeQuery(type_query) => type_query.into(),
       TsTypeLit(type_literal) => type_literal.into(),
-      _ => TsTypeDef {
-        repr: "<UNIMPLEMENTED>".to_string(),
-        ..Default::default()
-      },
+      TsArrayType(array_type) => array_type.into(),
+      TsTupleType(tuple_type) => tuple_type.into(),
+      TsOptionalType(optional_type) => optional_type.into(),
+      TsRestType(rest_type) => rest_type.into(),
+      TsUnionOrIntersectionType(union_or_inter) => union_or_inter.into(),
+      TsConditionalType(conditional_type) => conditional_type.into(),
+      TsInferType(infer_type) => infer_type.into(),
+      TsParenthesizedType(paren_type) => paren_type.into(),
+      TsTypeOperator(type_op_type) => type_op_type.into(),
+      TsIndexedAccessType(indexed_access_type) => indexed_access_type.into(),
+      TsMappedType(mapped_type) => mapped_type.into(),
+      TsLitType(lit_type) => lit_type.into(),
+      TsTypePredicate(type_predicate_type) => type_predicate_type.into(),
+      TsImportType(import_type) => import_type.into(),
     }
   }
 }
@@ -596,11 +640,12 @@ pub struct TsTypeRefDef {
   pub type_name: String,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
 pub enum LiteralDefKind {
   Number,
   String,
+  Template,
   Boolean,
   BigInt,
 }
@@ -616,6 +661,9 @@ pub struct LiteralDef {
   #[serde(skip_serializing_if = "Option::is_none")]
   pub string: Option<String>,
 
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub ts_types: Option<Vec<TsTypeDef>>,
+
   #[serde(skip_serializing_if = "Option::is_none")]
   pub boolean: Option<bool>,
 }
@@ -645,6 +693,22 @@ pub struct TsConditionalDef {
   pub false_type: Box<TsTypeDef>,
 }
 
+#[derive(Debug, Serialize, Deserialize, Clone)]
+#[serde(rename_all = "camelCase")]
+pub struct TsInferDef {
+  pub type_param: Box<TsTypeParamDef>,
+}
+
+#[derive(Debug, Serialize, Deserialize, Clone)]
+#[serde(rename_all = "camelCase")]
+pub struct TsImportTypeDef {
+  pub specifier: String,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub qualifier: Option<String>,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub type_params: Option<Vec<TsTypeDef>>,
+}
+
 #[derive(Debug, Serialize, Deserialize, Clone)]
 #[serde(rename_all = "camelCase")]
 pub struct TsIndexedAccessDef {
@@ -653,6 +717,32 @@ pub struct TsIndexedAccessDef {
   pub index_type: Box<TsTypeDef>,
 }
 
+/// Mapped Types
+///
+/// ```ts
+/// readonly [Properties in keyof Type as NewType]: Type[Properties]
+/// ```
+///
+/// - `readonly` = `TruePlusMinus::True`
+/// - `type_param` = `Some(TsTypeParamDef)` (`Properties in keyof Type`)
+/// - `name_type` = `Some(TsTypeDef)` (`NewType`)
+/// - `optional` = `None`
+/// - `ts_type` = `Some(TsTypeDef)` (`Type[Properties]`)
+///
+#[derive(Debug, Serialize, Deserialize, Clone)]
+#[serde(rename_all = "camelCase")]
+pub struct TsMappedTypeDef {
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub readonly: Option<TruePlusMinus>,
+  pub type_param: Box<TsTypeParamDef>,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub name_type: Option<Box<TsTypeDef>>,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub optional: Option<TruePlusMinus>,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub ts_type: Option<Box<TsTypeDef>>,
+}
+
 #[derive(Debug, Serialize, Deserialize, Clone)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralMethodDef {
@@ -766,9 +856,12 @@ pub enum TsTypeDefKind {
   This,
   FnOrConstructor,
   Conditional,
+  Infer,
   IndexedAccess,
+  Mapped,
   TypeLiteral,
   TypePredicate,
+  ImportType,
 }
 
 #[derive(Debug, Default, Serialize, Deserialize, Clone)]
@@ -823,14 +916,23 @@ pub struct TsTypeDef {
   #[serde(skip_serializing_if = "Option::is_none")]
   pub conditional_type: Option<TsConditionalDef>,
 
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub infer: Option<TsInferDef>,
+
   #[serde(skip_serializing_if = "Option::is_none")]
   pub indexed_access: Option<TsIndexedAccessDef>,
 
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub mapped_type: Option<TsMappedTypeDef>,
+
   #[serde(skip_serializing_if = "Option::is_none")]
   pub type_literal: Option<TsTypeLiteralDef>,
 
   #[serde(skip_serializing_if = "Option::is_none")]
   pub type_predicate: Option<TsTypePredicateDef>,
+
+  #[serde(skip_serializing_if = "Option::is_none")]
+  pub import_type: Option<TsImportTypeDef>,
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone)]
@@ -888,68 +990,135 @@ impl Display for TsTypePredicateDef {
   }
 }
 
+fn get_span_from_type(ts_type: &TsType) -> Span {
+  use deno_ast::swc::ast::TsType::*;
+
+  match ts_type {
+    TsArrayType(ref t) => get_span_from_type(t.elem_type.as_ref()),
+    TsConditionalType(ref t) => get_span_from_type(t.check_type.as_ref()),
+    TsFnOrConstructorType(ref t) => {
+      if let Some(t) = t.clone().ts_constructor_type() {
+        t.span
+      } else if let Some(t) = t.clone().ts_fn_type() {
+        t.span
+      } else {
+        unreachable!("no type found")
+      }
+    }
+    TsImportType(ref t) => t.span,
+    TsIndexedAccessType(ref t) => get_span_from_type(&t.index_type),
+    TsInferType(t) => t.span,
+    TsKeywordType(t) => t.span,
+    TsLitType(t) => t.span,
+    TsMappedType(t) => t.span,
+    TsOptionalType(t) => t.span,
+    TsParenthesizedType(t) => t.span,
+    TsRestType(t) => t.span,
+    TsThisType(t) => t.span,
+    TsTupleType(t) => t.span,
+    TsTypeLit(t) => t.span,
+    TsTypeOperator(t) => t.span,
+    TsTypePredicate(t) => t.span,
+    TsTypeQuery(t) => t.span,
+    TsTypeRef(t) => t.span,
+    TsUnionOrIntersectionType(t) => {
+      if let Some(t) = t.clone().ts_intersection_type() {
+        t.span
+      } else if let Some(t) = t.clone().ts_union_type() {
+        t.span
+      } else {
+        unreachable!("no type found")
+      }
+    }
+  }
+}
+
 impl TsTypeDef {
-  pub fn number_literal(num: &Number) -> TsTypeDef {
+  pub fn number_literal(num: &Number) -> Self {
     let repr = format!("{}", num.value);
     let lit = LiteralDef {
       kind: LiteralDefKind::Number,
       number: Some(num.value),
       string: None,
+      ts_types: None,
       boolean: None,
     };
     Self::literal(repr, lit)
   }
 
-  pub fn string_literal(str_: &Str) -> TsTypeDef {
+  pub fn string_literal(str_: &Str) -> Self {
     let repr = str_.value.to_string();
     let lit = LiteralDef {
       kind: LiteralDefKind::String,
       number: None,
       string: Some(str_.value.to_string()),
+      ts_types: None,
       boolean: None,
     };
     Self::literal(repr, lit)
   }
 
-  pub fn tpl_literal(quasis: &[TplElement]) -> TsTypeDef {
-    // A template literal in a type is not allowed to have
-    // expressions, so there will only be one quasi.
-    let quasi = quasis.get(0).expect("Expected tpl to have a quasi.");
-    let text = quasi.raw.value.to_string();
-    let repr = text.clone();
+  pub fn tpl_literal(types: &[Box<TsType>], quasis: &[TplElement]) -> Self {
+    let mut ts_types: Vec<(Span, Self, String)> = Vec::new();
+    for ts_type in types {
+      let t: Self = ts_type.as_ref().into();
+      let repr = format!("${{{}}}", t);
+      ts_types.push((get_span_from_type(ts_type), t, repr))
+    }
+    for quasi in quasis {
+      let repr = quasi.raw.value.to_string();
+      let lit = LiteralDef {
+        kind: LiteralDefKind::String,
+        number: None,
+        string: Some(repr.clone()),
+        ts_types: None,
+        boolean: None,
+      };
+      ts_types.push((quasi.span, Self::literal(repr.clone(), lit), repr));
+    }
+    ts_types.sort_by(|(a, _, _), (b, _, _)| a.cmp(b));
+    let repr = ts_types
+      .iter()
+      .map(|(_, _, s)| s.as_str())
+      .collect::<Vec<&str>>()
+      .join("");
+    let ts_types = Some(ts_types.into_iter().map(|(_, t, _)| t).collect());
     let lit = LiteralDef {
-      kind: LiteralDefKind::String, // semantically the same
+      kind: LiteralDefKind::Template,
       number: None,
-      string: Some(text),
+      string: None,
+      ts_types,
       boolean: None,
     };
     Self::literal(repr, lit)
   }
 
-  pub fn bool_literal(bool_: &Bool) -> TsTypeDef {
+  pub fn bool_literal(bool_: &Bool) -> Self {
     let repr = bool_.value.to_string();
     let lit = LiteralDef {
       kind: LiteralDefKind::Boolean,
       number: None,
       string: None,
+      ts_types: None,
       boolean: Some(bool_.value),
     };
     Self::literal(repr, lit)
   }
 
-  pub fn bigint_literal(bigint_: &BigInt) -> TsTypeDef {
+  pub fn bigint_literal(bigint_: &BigInt) -> Self {
     let repr = bigint_.value.to_string();
     let lit = LiteralDef {
       kind: LiteralDefKind::BigInt,
       number: None,
       string: Some(bigint_.value.to_string()),
+      ts_types: None,
       boolean: None,
     };
     Self::literal(repr, lit)
   }
 
-  pub fn regexp(repr: String) -> TsTypeDef {
-    TsTypeDef {
+  pub fn regexp(repr: String) -> Self {
+    Self {
       repr,
       kind: Some(TsTypeDefKind::TypeRef),
       type_ref: Some(TsTypeRefDef {
@@ -960,28 +1129,28 @@ impl TsTypeDef {
     }
   }
 
-  pub fn keyword(keyword_str: &str) -> TsTypeDef {
+  pub fn keyword(keyword_str: &str) -> Self {
     Self::keyword_with_repr(keyword_str, keyword_str)
   }
 
-  pub fn number_with_repr(repr: &str) -> TsTypeDef {
+  pub fn number_with_repr(repr: &str) -> Self {
     Self::keyword_with_repr("number", repr)
   }
 
-  pub fn string_with_repr(repr: &str) -> TsTypeDef {
+  pub fn string_with_repr(repr: &str) -> Self {
     Self::keyword_with_repr("string", repr)
   }
 
-  pub fn bool_with_repr(repr: &str) -> TsTypeDef {
+  pub fn bool_with_repr(repr: &str) -> Self {
     Self::keyword_with_repr("boolean", repr)
   }
 
-  pub fn bigint_with_repr(repr: &str) -> TsTypeDef {
+  pub fn bigint_with_repr(repr: &str) -> Self {
     Self::keyword_with_repr("bigint", repr)
   }
 
-  pub fn keyword_with_repr(keyword_str: &str, repr: &str) -> TsTypeDef {
-    TsTypeDef {
+  pub fn keyword_with_repr(keyword_str: &str, repr: &str) -> Self {
+    Self {
       repr: repr.to_string(),
       kind: Some(TsTypeDefKind::Keyword),
       keyword: Some(keyword_str.to_string()),
@@ -989,8 +1158,8 @@ impl TsTypeDef {
     }
   }
 
-  fn literal(repr: String, lit: LiteralDef) -> TsTypeDef {
-    TsTypeDef {
+  fn literal(repr: String, lit: LiteralDef) -> Self {
+    Self {
       repr,
       kind: Some(TsTypeDefKind::Literal),
       literal: Some(lit),
@@ -1004,26 +1173,25 @@ pub fn ts_type_ann_to_def(type_ann: &TsTypeAnn) -> TsTypeDef {
 
   match &*type_ann.type_ann {
     TsKeywordType(keyword_type) => keyword_type.into(),
-    TsLitType(lit_type) => lit_type.into(),
-    TsTypeRef(type_ref) => type_ref.into(),
-    TsUnionOrIntersectionType(union_or_inter) => union_or_inter.into(),
-    TsArrayType(array_type) => array_type.into(),
-    TsTupleType(tuple_type) => tuple_type.into(),
-    TsTypeOperator(type_op_type) => type_op_type.into(),
-    TsParenthesizedType(paren_type) => paren_type.into(),
-    TsRestType(rest_type) => rest_type.into(),
-    TsOptionalType(optional_type) => optional_type.into(),
-    TsTypeQuery(type_query) => type_query.into(),
     TsThisType(this_type) => this_type.into(),
     TsFnOrConstructorType(fn_or_con_type) => fn_or_con_type.into(),
-    TsConditionalType(conditional_type) => conditional_type.into(),
-    TsIndexedAccessType(indexed_access_type) => indexed_access_type.into(),
+    TsTypeRef(type_ref) => type_ref.into(),
+    TsTypeQuery(type_query) => type_query.into(),
     TsTypeLit(type_literal) => type_literal.into(),
+    TsArrayType(array_type) => array_type.into(),
+    TsTupleType(tuple_type) => tuple_type.into(),
+    TsOptionalType(optional_type) => optional_type.into(),
+    TsRestType(rest_type) => rest_type.into(),
+    TsUnionOrIntersectionType(union_or_inter) => union_or_inter.into(),
+    TsConditionalType(conditional_type) => conditional_type.into(),
+    TsInferType(infer_type) => infer_type.into(),
+    TsParenthesizedType(paren_type) => paren_type.into(),
+    TsTypeOperator(type_op_type) => type_op_type.into(),
+    TsIndexedAccessType(indexed_access_type) => indexed_access_type.into(),
+    TsMappedType(mapped_type) => mapped_type.into(),
+    TsLitType(lit_type) => lit_type.into(),
     TsTypePredicate(type_predicate) => type_predicate.into(),
-    _ => TsTypeDef {
-      repr: "<TODO>".to_string(),
-      ..Default::default()
-    },
+    TsImportType(import_type) => import_type.into(),
   }
 }
 
@@ -1142,13 +1310,12 @@ fn infer_ts_type_from_call_expr(call_expr: &CallExpr) -> Option<TsTypeDef> {
 }
 
 fn infer_ts_type_from_tpl(tpl: &Tpl, is_const: bool) -> TsTypeDef {
+  // TODO(@kitsonk) we should iterate over the expr and if each expr has a
+  // ts_type or can be trivially inferred, it should be passed to the
+  // tp_literal
   if tpl.quasis.len() == 1 && is_const {
-    // If no placeholder is present, the type can be inferred.
-    //   e.g.) const tpl = `Hello world!`; // tpl has the type of `"Hello world!"`.
-    TsTypeDef::tpl_literal(&tpl.quasis)
+    TsTypeDef::tpl_literal(&[], &tpl.quasis)
   } else {
-    // If placeholders are present, the type cannot be inferred.
-    //   e.g.) const tpl = `Hello ${name}!`; // tpl has the type of `string`.
     TsTypeDef::string_with_repr("string")
   }
 }
@@ -1156,7 +1323,7 @@ fn infer_ts_type_from_tpl(tpl: &Tpl, is_const: bool) -> TsTypeDef {
 impl Display for TsTypeDef {
   fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
     if self.kind.is_none() {
-      return write!(f, "{}", colors::red("<UNIMPLEMENTED>"));
+      return write!(f, "{}", colors::red("[UNSUPPORTED]"));
     }
 
     let kind = self.kind.as_ref().unwrap();
@@ -1174,6 +1341,21 @@ impl Display for TsTypeDef {
           &*conditional.false_type
         )
       }
+      TsTypeDefKind::Infer => {
+        let infer = self.infer.as_ref().unwrap();
+        write!(f, "{} {}", colors::magenta("infer"), infer.type_param)
+      }
+      TsTypeDefKind::ImportType => {
+        let import_type = self.import_type.as_ref().unwrap();
+        write!(f, "import(\"{}\")", import_type.specifier)?;
+        if let Some(qualifier) = &import_type.qualifier {
+          write!(f, ".{}", qualifier)?;
+        }
+        if let Some(type_params) = &import_type.type_params {
+          write!(f, "<{}>", SliceDisplayer::new(type_params, ", ", false))?;
+        }
+        Ok(())
+      }
       TsTypeDefKind::FnOrConstructor => {
         let fn_or_constructor = self.fn_or_constructor.as_ref().unwrap();
         write!(
@@ -1200,6 +1382,48 @@ impl Display for TsTypeDef {
         let intersection = self.intersection.as_ref().unwrap();
         write!(f, "{}", SliceDisplayer::new(intersection, " & ", false))
       }
+      TsTypeDefKind::Mapped => {
+        let mapped_type = self.mapped_type.as_ref().unwrap();
+        let readonly = match mapped_type.readonly {
+          Some(TruePlusMinus::True) => {
+            format!("{} ", colors::magenta("readonly"))
+          }
+          Some(TruePlusMinus::Plus) => {
+            format!("+{} ", colors::magenta("readonly"))
+          }
+          Some(TruePlusMinus::Minus) => {
+            format!("-{} ", colors::magenta("readonly"))
+          }
+          _ => "".to_string(),
+        };
+        let optional = match mapped_type.optional {
+          Some(TruePlusMinus::True) => "?",
+          Some(TruePlusMinus::Plus) => "+?",
+          Some(TruePlusMinus::Minus) => "-?",
+          _ => "",
+        };
+        let type_param =
+          if let Some(ts_type_def) = &mapped_type.type_param.constraint {
+            format!("{} in {}", mapped_type.type_param.name, ts_type_def)
+          } else {
+            mapped_type.type_param.to_string()
+          };
+        let name_type = if let Some(name_type) = &mapped_type.name_type {
+          format!(" {} {}", colors::magenta("as"), name_type)
+        } else {
+          "".to_string()
+        };
+        let ts_type = if let Some(ts_type) = &mapped_type.ts_type {
+          format!(": {}", ts_type)
+        } else {
+          "".to_string()
+        };
+        write!(
+          f,
+          "{}[{}{}]{}{}",
+          readonly, type_param, name_type, optional, ts_type
+        )
+      }
       TsTypeDefKind::Keyword => {
         write!(f, "{}", colors::cyan(self.keyword.as_ref().unwrap()))
       }
@@ -1216,6 +1440,31 @@ impl Display for TsTypeDef {
             "{}",
             colors::green(&format!("\"{}\"", literal.string.as_ref().unwrap()))
           ),
+          LiteralDefKind::Template => {
+            write!(f, "{}", colors::green("`"))?;
+            for ts_type in literal.ts_types.as_ref().unwrap() {
+              let kind = ts_type.kind.as_ref().unwrap();
+              if *kind == TsTypeDefKind::Literal {
+                let literal = ts_type.literal.as_ref().unwrap();
+                if literal.kind == LiteralDefKind::String {
+                  write!(
+                    f,
+                    "{}",
+                    colors::green(literal.string.as_ref().unwrap())
+                  )?;
+                  continue;
+                }
+              }
+              write!(
+                f,
+                "{}{}{}",
+                colors::magenta("${"),
+                ts_type,
+                colors::magenta("}")
+              )?;
+            }
+            write!(f, "{}", colors::green("`"))
+          }
           LiteralDefKind::Number => write!(
             f,
             "{}",
